package com.fastbee.project.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.fastbee.common.constant.ProjectLevelConstant;
import com.fastbee.common.core.domain.entity.SysDept;
import com.fastbee.common.core.domain.entity.SysUser;
import com.fastbee.common.exception.ServiceException;
import com.fastbee.common.holder.ProjectHolder;
import com.fastbee.common.utils.DateUtils;
import com.fastbee.project.domain.Project;
import com.fastbee.project.mapper.ProjectMapper;
import com.fastbee.project.service.IProjectService;

import com.fastbee.system.domain.SysDistrict;
import com.fastbee.system.mapper.SysDeptMapper;
import com.fastbee.system.mapper.SysDistrictMapper;
import com.fastbee.system.mapper.SysUserMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 项目Service业务层处理
 * @author kerwincui
 * &#064;date  2024-09-26
 */
@Service
public class ProjectServiceImpl implements IProjectService
{

        private final ProjectMapper projectMapper;
        private final SysDeptMapper sysDeptMapper;
        private final SysUserMapper sysUserMapper;

        private final SysDistrictMapper sysDistrictMapper;

        public ProjectServiceImpl(ProjectMapper projectMapper, SysDeptMapper sysDeptMapper, SysUserMapper sysUserMapper, SysDistrictMapper sysDistrictMapper) {
                this.projectMapper = projectMapper;
                this.sysDeptMapper = sysDeptMapper;
                this.sysUserMapper = sysUserMapper;
                this.sysDistrictMapper = sysDistrictMapper;
        }

        /**
         * 查询项目
         *
         * @param projectId 项目主键
         * @return 项目
         */
        @Override
        public Project selectProjectByProjectId(Long projectId)
        {
                return projectMapper.selectProjectById(projectId);
        }

        /**
         * 查询项目列表
         */
        @Override
        public List<Project> selectProjectList(Project project)
        {
                return  new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getId, Project::getProjectName, Project::getScope,
                                Project::getDeptId,
                                Project::getLogo, Project::getImage, Project::getLevel, Project::getDeptName,
                                Project::getCreateTime,
                                Project::getAdministrativeAreaCode,Project::getAdministrativeArea, Project::getProvinceCode,Project::getCityCode,Project::getCountyCode)
                        .like(StringUtils.isNotBlank(project.getSearchValue()),Project::getProjectName, project.getSearchValue())
                        .list();
        }

        /**
         * 新增项目
         */
        @Override
        public int insertProject(Project project)
        {
                //判断选择的机构下是否已经绑定项目
               List<Project> projectList = new LambdaQueryChainWrapper<>(projectMapper)
                       .select(Project::getId, Project::getProjectName)
                       .eq(Project::getDeptId, project.getDeptId()).list();
               if(!projectList.isEmpty()){
                       throw new ServiceException("该机构下已经存在项目");
               }
                //项目行政区域处理
                //拆解项目行政区域代码信息
                if(StringUtils.isBlank(project.getAdministrativeAreaCode())){
                        throw new ServiceException("项目行政区域代码不能为空！");
                }
                //判断行政区域代码json格式
                if(isNotValidJsonArray(project.getAdministrativeAreaCode())){
                        throw new ServiceException("项目行政区域代码多选列表非json数组！");
                }
                //项目省代码列表
                List<String> provincialCodeList =new ArrayList<>();
                //项目市代码列表
                List<String> cityCodeList =new ArrayList<>();
                //项目县代码列表
                List<String> countyCodeList =new ArrayList<>();
                //项目镇代码列表
                List<String> townCodeList =new ArrayList<>();
                //项目村代码列表
                List<String> villageCodeList =new ArrayList<>();
                JSONArray administrativeAreaList = JSONUtil.parseArray(project.getAdministrativeAreaCode());
                administrativeAreaList.forEach(item->{
                        if(isNotValidJsonArray(item.toString())){
                                throw new ServiceException("项目行政区域代码多选列表的元素非json数组！");
                        }
                        //单个行政区划代码列表
                        JSONArray administrativeArea = JSONUtil.parseArray(item);
                        //拆解出每个行政区划代码里面的镇或村
                         if(administrativeArea.size()!=5){
                                 throw new ServiceException("项目行政区域代码多选列表的元素长度必须为5，到达村级别！");
                         }
                         //保存项目省代码列表
                         provincialCodeList.add(administrativeArea.get(0).toString());
                         //保存项目市代码列表
                         cityCodeList.add(administrativeArea.get(1).toString());
                         //保存项目县代码列表
                         countyCodeList.add(administrativeArea.get(2).toString());
                         //保存项目镇代码列表
                         townCodeList.add(administrativeArea.get(3).toString());
                         //保存项目村代码列表
                         villageCodeList.add(administrativeArea.get(4).toString());
                });

                //设置项目省、市、县、镇、村行政区划代码列表
                project.setProvinceCode(JSONUtil.toJsonStr(provincialCodeList));
                project.setCityCode(JSONUtil.toJsonStr(cityCodeList));
                project.setCountyCode(JSONUtil.toJsonStr(countyCodeList));
                project.setTownCode(JSONUtil.toJsonStr(townCodeList));
                project.setVillageCode(JSONUtil.toJsonStr(villageCodeList));
                project.setCreateTime(DateUtils.getNowDate());
                return projectMapper.insertProject(project);
        }

        /**
         * 验证json对象是否是数组
         */
        public  boolean isNotValidJsonArray(String jsonStr) {
                try {
                        JSONUtil.parseArray(jsonStr);
                } catch (JSONException e) {
                        return true;
                }
                return false;
        }

        /**
         * 修改项目
         */
        @Override
        public int updateProject(Project project)
        {
                //判断选择的机构下是否已经绑定项目
                Project one = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getId, Project::getProjectName,Project::getDeptId)
                        .eq(Project::getDeptId, project.getDeptId())
//                        .ne(Project::getId,project.getId())
                        .one();
                if(Objects.isNull(one)){
                        throw new ServiceException("项目不存在！");
                }
                if(!one.getDeptId().equals(project.getDeptId())){
                        throw new ServiceException("非法修改项目机构！");
                }
                project.setUpdateTime(DateUtils.getNowDate());
                return projectMapper.updateProject(project);
        }

        /**
         * 删除项目
         */
        @Override
        public int deleteProjectByProjectIds(Long projectId)
        {
                return projectMapper.deleteById(projectId);
        }


        /**
         * 获取管理员所管理的项目id以及子项目id列表
         */
        @Override
        public Map<String, Object> selectProjectByUserId(Long userId) {
                //查询用户所属机构
                SysUser sysUser = new LambdaQueryChainWrapper<>(sysUserMapper)
                        .select(SysUser::getUserId, SysUser::getDeptId)
                        .eq(SysUser::getUserId, userId)
                        .one();


                //查询该管理员所管理的项目id以及子项目id列表
                Project project = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getId, Project::getProjectName,Project::getLevel,Project::getDeptId
                        ,Project::getCityCode,Project::getCountyCode)
                        .eq(Project::getDeptId, sysUser.getDeptId())
                        .one();
                if(Objects.isNull(project)){
                        return null;
                }
                List<Long> projects = getAllProjects(project.getId());
                //把列表进行Base64编码
                String projectsBase64 = Base64.encode(projects.toString());
                HashMap<String , Object> map = new HashMap<>();
                map.put("projectIds",projectsBase64);
                map.put("projectId",project.getId());
                map.put("projectName", project.getProjectName());
                if(project.getLevel().equals("市级")){
                        map.put("projectLevel", ProjectLevelConstant.LEVEL_CITY);
                        map.put("projectAreaCode", JSONUtil.parseArray(project.getCityCode()).get(0));
                }
                if(project.getLevel().equals("县级")){
                        map.put("projectLevel",  ProjectLevelConstant.LEVEL_COUNTY);
                        map.put("projectAreaCode",JSONUtil.parseArray(project.getCountyCode()).get(0));

                }

                return map;
        }

        /**
         * 查询未绑定项目的部门列表
         */
        @Override
        public List<SysDept> getUnbindDeptList() {
                //查询所有机构
                List<SysDept> sysDeptList = new LambdaQueryChainWrapper<>(sysDeptMapper)
                        .select(SysDept::getDeptId, SysDept::getDeptName)
                        .list();
                //查询所有项目
                List<Project> projectList = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getId, Project::getDeptId)
                        .list();
                //遍历项目，把已绑定项目的机构从机构列表中移除
                for (Project project : projectList) {
                        if (Objects.nonNull(project.getDeptId())) {
                            sysDeptList.removeIf(sysDept -> sysDept.getDeptId().equals(project.getDeptId()));
                        }
                }
                return sysDeptList;
        }

        /**
         * 查询管理员可管理项目下拉框列表
         */
        @Override
        public List<Project> selectAdminProjectList() {
                //判断管理员级别
                List<Project> projectList=new ArrayList<>();
                //超级管理员
                if(Objects.isNull(ProjectHolder.getProjectInfo().getProjectIdList())){
                        //查询所有一级项目列表
                        projectList = new LambdaQueryChainWrapper<>(projectMapper)
                                .select(Project::getId, Project::getProjectName)
                                .isNull(Project::getParentId)
                                .list();
                }
                //市级/县级项目
                else if(Objects.nonNull(ProjectHolder.getProjectInfo().getProjectIdList())){
                        //查询所有子一级项目列表
                        projectList = new LambdaQueryChainWrapper<>(projectMapper)
                                .select(Project::getId, Project::getProjectName)
                                .in(!ProjectHolder.getProjectInfo().getProjectIdList().isEmpty(),Project::getParentId,
                                        ProjectHolder.getProjectInfo().getProjectIdList())
                                .list();
                }
                return projectList;
        }
        /**
         * 查询项目的行政区划树状列表
         */
        @Override
        public List<SysDistrict> getProjectAreaTree(Long projectId,Integer startLevel) {
                //参数校验
                if(startLevel>5||startLevel<1){
                        throw new ServiceException("超出行政级别查询范围(1~5级)!");
                }
                //查询项目的行政区划信息
                Project project = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProvinceCode, Project::getCityCode, Project::getCountyCode,
                                Project::getTownCode, Project::getVillageCode)
                        .eq(Project::getId, projectId)
                        .one();
                if (Objects.isNull(project)) {
                        throw new ServiceException("项目不存在");
                }
                //判断项目的省、市、县、镇、村行政区划列表数据不为null
                isProvinceCodeBlank(project);

                List<Object> administrativeCodeList = getTownAndvillageCodeList(project,startLevel);

                //查询项目所属县下面的所有镇、村行政区划列表
                List<SysDistrict> projectDistrictList;
                LambdaQueryChainWrapper<SysDistrict> sysDistrictLambdaQueryChainWrapper = new LambdaQueryChainWrapper<>(sysDistrictMapper)
                        .select(SysDistrict::getId, SysDistrict::getParentId, SysDistrict::getName, SysDistrict::getAdcode,
                                SysDistrict::getLevel, SysDistrict::getFullName, SysDistrict::getName
                                , SysDistrict::getLng, SysDistrict::getLat);

                //限制行政区划级别
                projectDistrictList = sysDistrictLambdaQueryChainWrapper
                       .in(SysDistrict::getAdcode,administrativeCodeList)
                        .list();

//                System.err.println("项目所属县下面的所有镇、村行政区划列表");
//                System.err.println(projectDistrictList);
                //构建树状结构
            return buildTree(projectDistrictList);
        }

        private static List<Object> getTownAndvillageCodeList(Project project,Integer startLevel) {
                //查询项目的省、市、县、镇、村行政区划列表数据
                //目前项目下面的镇或者村都可以多选划分成一个项目
                //找到项目行政区划开始多选的级别
                //判断省是否多选
                JSONArray provinceCodeList = JSONUtil.parseArray(project.getProvinceCode());
                //判断集合中的元素值都相等则说明省未多选
                if (!provinceCodeList.stream().allMatch(provinceCode -> provinceCode.equals(provinceCodeList.get(0)))) {
                        throw new ServiceException("项目省级行政区划只能包含一个省！");
                }
                //省去重
                List<Object> distinctProvinceCodeList = provinceCodeList.stream().distinct().collect(Collectors.toList());

                //判断市是否多选
                JSONArray cityCodeList = JSONUtil.parseArray(project.getCityCode());
                //判断集合中的元素值都相等则说明市未多选
                if (!cityCodeList.stream().allMatch(cityCode -> cityCode.equals(cityCodeList.get(0)))) {
                        throw new ServiceException("项目市级行政区划只能包含一个市！");
                }
                //市去重
                List<Object> distinctCityCodeList = cityCodeList.stream().distinct().collect(Collectors.toList());

                //判断县是否多选
                JSONArray countyCodeList = JSONUtil.parseArray(project.getCountyCode());
                //判断集合中的元素值都相等则说明县未多选
                if (!countyCodeList.stream().allMatch(countyCode -> countyCode.equals(countyCodeList.get(0)))) {
                        throw new ServiceException("项目县级行政区划只能包含一个县！");
                }
                //县去重
                List<Object>  distinctcCountyCodeList= countyCodeList.stream().distinct().collect(Collectors.toList());

                //判断镇是否多选
                JSONArray townCodeList = JSONUtil.parseArray(project.getTownCode());
                //判断集合中的元素值都相等则说明镇未多选
                List<Object> distinctTownCodeList;

                distinctTownCodeList = townCodeList.stream().distinct().collect(Collectors.toList());
                JSONArray villageCodeList = JSONUtil.parseArray(project.getVillageCode());

//                System.err.println("镇行政区划代码");
//                System.err.println(distinctTownCodeList);
//                System.err.println("村行政区划代码");
//                System.err.println(villageCodeList);
                //合并省、市、县、镇、村行政区划代码集合
                List<Object> administrativeCodeList = new ArrayList<>();
                int[] startLevels = {1, 2, 3, 4, 5};
                List<List<Object>> distinctCodeLists = Arrays.asList(distinctProvinceCodeList, distinctCityCodeList, distinctcCountyCodeList, distinctTownCodeList, villageCodeList);

                for (int i = 0; i < startLevels.length; i++) {
                        if (startLevel <= startLevels[i]) {
                                for (int j = i; j < distinctCodeLists.size(); j++) {
                                        administrativeCodeList.addAll(distinctCodeLists.get(j));
                                }
                                break;
                        }
                }
//                System.err.println("合并后镇、村行政区划代码");
//                System.err.println(administrativeCodeList);
                return administrativeCodeList;
        }

        private static void isProvinceCodeBlank(Project project) {
                if (StringUtils.isBlank(project.getProvinceCode())){
                        throw new ServiceException("项目省级行政区划信息缺失！");
                }
                if (StringUtils.isBlank(project.getCityCode())){
                        throw new ServiceException("项目市级行政区划信息缺失！");
                }
                if (StringUtils.isBlank(project.getCountyCode())){
                        throw new ServiceException("项目县级行政区划信息缺失！");
                }
                if (StringUtils.isBlank(project.getTownCode())){
                        throw new ServiceException("项目镇级行政区划信息缺失！");
                }
                if (StringUtils.isBlank(project.getVillageCode())){
                        throw new ServiceException("项目村级行政区划信息缺失！");
                }
        }

        private List<SysDistrict> buildTree(List<SysDistrict> SysDistricts) {
                Map<Long, SysDistrict> map = new HashMap<>();
                List<SysDistrict> roots = new ArrayList<>();
                for (SysDistrict sysDistrict : SysDistricts) {
                        map.put(sysDistrict.getAdcode(), sysDistrict);
                }
                return getDistricts(SysDistricts, map, roots);
        }

        private List<SysDistrict> getDistricts(List<SysDistrict> SysDistricts, Map<Long, SysDistrict> map, List<SysDistrict> roots) {
                for (SysDistrict sysDistrict : SysDistricts) {
                        Long parentId = sysDistrict.getParentId();
                        if (parentId == null || !map.containsKey(parentId)) {
                                roots.add(sysDistrict);
                        } else {
                                SysDistrict parent = map.get(parentId);
                                if (parent.getChildren() == null) {
                                        parent.setChildren(new ArrayList<>());
                                }
                                parent.getChildren().add(sysDistrict);
                        }
                }
                return roots;
        }

        public List<Long> getAllProjects(Long projectId) {
                List<Long> allProjects = new ArrayList<>();

                // 查询当前项目
                Project currentProject = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getId, Project::getProjectName)
                        .eq(Project::getId, projectId)
                        .one();

                if (currentProject != null) {
                        allProjects.add(currentProject.getId());

                        // 查询当前项目的子项目
                        List<Project> children = new LambdaQueryChainWrapper<>(projectMapper)
                                .select(Project::getId, Project::getProjectName)
                                .eq(Project::getParentId, projectId)
                                .list();

                        // 递归查询子项目
                        for (Project child : children) {
                                allProjects.addAll(getAllProjects(child.getId()));
                        }
                }

                return allProjects;
        }
}

